/* ******************************************************************************************** */
/* *                                                                     		              * */
/* *                              X.25 Interface Library                                      * */
/* *                              ----------------------                                      * */
/* *                                                                     		              * */
/* *                                   Example_1.c                                            * */
/* *                                                                     		              * */
/* *..........................................................................................* */
/* *                                                                                          * */
/* *     Example code for developers                                                          * */
/* *                                                                                          * */
/* *                                                                                          * */
/* *------------------------------------------------------------------------------------------* */
/* *                                                                     		              * */
/* *                   Copyright  1988-1991 Apple Computer, Inc.              		          * */
/* *                               All Rights Reserved                        		          * */
/* *                           Apple Computer Confidential                     		          * */
/* *                                                                     		              * */
/* *------------------------------------------------------------------------------------------* */
/* *                                                                                          * */
/* *     Revision History:                                                                    * */
/* *                                                                                          * */
/* *                                                                                          * */
/* *     Date       Individual          Changes                                               * */
/* *     .........  ................... ..................................................    * */
/* *                                                                     		              * */
/* *                                                                                          * */
/* *     05 Jul 88  Rick Holzgrafe      Original version that was written to work with        * */
/* *                                    version 1.0d2 of the X.25 Interface Library.          * */
/* *                                                                                          * */
/* *     12 Apr 89  D. N. Willey        Major overhaul and re-write to work with version      * */
/* *                                    1.0d7 of the X.25 Interface Library.                  * */
/* *                                                                                          * */
/* *     16 Apr 91  D. N. Willey        Minor upgrade for release 1.0.1 of MacX25.            * */
/* *                                                                                          * */
/* *                                                                                          * */
/* ******************************************************************************************** */
    

/* ******************************************************************************************** */
/* *                                                                     		              * */
/* *	This file is sample code for developers showing the use of the X.25 Interface         * */
/* *    Library.                                                                	          * */
/* *                                                                     		              * */
/* *    It is the first example, and shows the steps necessary to open an X.25 SVC            * */
/* *    (Switched Virtual Circuit), exchange data over the circut, and then close the         * */
/* *    SVC and cleanup.                                                                 	  * */
/* *                                                                     		              * */
/* *                                                                     		              * */
/* *    	#	It can be compiled into an MPW Tool using these commands:                     * */
/* *                                                                                          * */
/* *    	C X25_Example_1.c                                                                 * */
/* *        Link -t MPST -c 'MPS ' -o X25_Example_1 		                                 * */
/* *        	X25_Example_1.c.o							                                 * */
/* *        	IPCGlue.o									                                 * */
/* *        	FSDES.o										                                 * */
/* *        	{CLibraries}CRunTime.o						                                 * */
/* *        	{CLibraries}CInterface.o					                                 * */
/* *        	{Libraries}Interface.o						                         		  * */
/* *        	{CLibraries}StdCLib.o                                                         * */
/* *        Rez -a -o :X25_Example_1						                                 * */
/* *            -i  "{MPW}Interfaces:RIncludes:"                                             * */
/* *            X25_Interface_Library.r                                                       * */
/* *                                                                                          * */
/* *                                                                                          * */
/* *        #                                                                                 * */
/* *        #   The X25_Example takes up to three arguments, with the first                   * */
/* *        #   argument being required. The example given here has only                      * */
/* *        #   this required first argument (Keep the part that says                         * */
/* *        #   "CxxxxxxDlistener e" and substitute your own DTE address.)                    * */
/* *        #                                                                                 * */
/* *        #   The second and third arguments are strings giving the                         * */
/* *        #   user's Name and Password to be used.  (These arguments are only               * */
/* *        #   required if "guest" login is not allowed to the server.)                      * */
/* *        #                                                                                 * */
/* *                                                                                          * */
/* *                                                                                          * */
/* *        #                                                                                 * */
/* *        #	This call should work correctly                                               * */
/* *        #                                                                                 * */
/* *        X25_Example_1 "C311040800433Dlistener e"                                          * */
/* *                                                                                          * */
/* *                                                                                          * */
/* *        #                                                                                 * */
/* *        #	This call will fail during call placement                                     * */
/* *        #                                                                                 * */
/* *        X25_Example_1 "C311040800433"                                                     * */
/* *                                                                                          * */
/* *                                                                                          * */
/* *        #                                                                                 * */
/* *        #	The following call will give read timeouts                                    * */
/* *        #                                                                                 * */
/* *        X25_Example_1 "C311040800433Dlistener"                                            * */
/* *                                                                                          * */
/* *                                                                                          * */
/* *	This is only an example, not a useful tool nor a particularly well-designed one.      * */
/* *    It establishes a single X.25 circuit using a hard-coded X.25 address in ascii         * */
/* *    format, performs several reads and writes using different techniques, then shuts      * */
/* *    down the circuit. The peer process is assumed to be an "echo-er" that will simply     * */
/* *    reflect all our packets back to us.                                                   * */
/* *                                                                                          * */
/* *                                                                                          * */
/* ******************************************************************************************** */






/* ******************************************************************************************** */
/* *                                                                                          * */
/* *    Include all Common Incluce Files and Definitions                                      * */
/* *                                                                                          * */
/* ******************************************************************************************** */

# include <Memory.h>
# include <Quickdraw.h>
# include <stdio.h>

# include <X25_Interface.h>
# include <X25_Packet.h>


/* ******************************************************************************************** */
/* *                                                                                          * */
/* *    Defines and Typedefs                                                                  * */
/* *                                                                                          * */
/* ******************************************************************************************** */

# define MY_BUF_SZ		512


/*
 * Possible values for theReadStatus
 */
# define READ_IS_PENDING	-1
# define READ_WAS_OK		0
# define READ_FAILED		1


/* ******************************************************************************************** */
/* *                                                                                          * */
/* *    Internal Routine Definitions                                                          * */
/* *                                                                                          * */
/* ******************************************************************************************** */

static void		MyComplete        	( tcb_def *              	);
static void		error_print       	( tcb_def *, char *      	);

static OSErr	library_open 		(  							);
static OSErr	vccb_open 			(  							);
static OSErr	server_open 		(  							);
static OSErr	user_validate 		(  							);
static OSErr	x25_open 			(  							);
static OSErr	data_test 			(  							);
static OSErr	data_test_1 		(  							);
static OSErr	data_test_2 		(  							);
static OSErr	data_test_3 		(  							);
	
extern char *	c2pstr 				(	char *					);
	

/* ******************************************************************************************** */
/* *                                                                                          * */
/* *    Global Data Area                                                                      * */
/* *                                                                                          * */
/* ******************************************************************************************** */

	tcb_def					tcb;							/* Control Block for one VC		    */
	tcb_def		*	const	tcb_ptr = &tcb;					/* Ptr to Control Block for one VC	*/
	
	char		 			empty_string[]	= "\p";			/* Empty Text String 				*/

	char		*			connect_rec_ptr;				/* Pointer to Connection Record		*/
	char		*			user_name_ptr;					/* Pointer to User's Name			*/
	char		*			user_password_ptr;				/* Pointer to User's Password		*/

	char					write_buf     [MY_BUF_SZ+64];	/* Buffer for outgoing packets		*/
	char					read_buf      [MY_BUF_SZ+64];	/* Buffer for incoming packets		*/

	char					aux_write_buf [MY_BUF_SZ+64];	/* Buffer for outgoing data			*/
	char					aux_read_buf  [MY_BUF_SZ+64];	/* Buffer for incoming data			*/

	int 					theReadStatus;					/* Global variable, for 			*/
															/* communication between main() 	*/
															/* and MyComplete()					*/



/* ******************************************************************************************** */
/* *                                                                                          * */
/* *                                                                                          * */
/* *                                                                                          * */
/* ******************************************************************************************** */

main ( int argc, char *argv[] )
	{
	
	OSErr		exit_code;								/* Holds error returns from routines	*/
	
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Make the Macintosh Toolbox Initialization calls.  Remember that we are running as     | */
/* |    an MPW tool, some initialization calls have already been done by MPW.                 | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	MoreMasters 	(             );
	MoreMasters 	(             );
	InitGraf    	( &qd.thePort );
	InitCursor  	(             );


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Collect the input parameters passed to this program                                   | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	if ( argc < 2 )
		{
		printf ( "Usage: %s X.25_Address_String User_Name_string User_Password String \n\n",
				 argv[0] );
		exit ( 1 );
		}
	
	if ( argc >= 2 )
		connect_rec_ptr 	= argv[1];
	else
		connect_rec_ptr 	= empty_string;
		
	
	if ( argc >= 3 )		
		user_name_ptr 		= argv[2];
	else
		user_name_ptr	 	= empty_string;
	
	if ( argc >= 4 )
		user_password_ptr 	= argv[3];
	else
		user_password_ptr 	= empty_string;
	
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Convert the input strings from "C" form to "Pascal" form.                             | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	c2pstr ( connect_rec_ptr   );
	c2pstr ( user_name_ptr     );
	c2pstr ( user_password_ptr );


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Get setup for the processing                                                          | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	printf ( "                                                                              \n" );
	printf ( "#---------------------------------------------------------------------------- \n" );
	printf ( "#                                                                             \n" );


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Call the first routine in the chain                                                   | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	exit_code = library_open ( );
	

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Cleanup and Exit                                                                      | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	
	printf ( "#                                                                             \n" );
	printf ( "#---------------------------------------------------------------------------- \n" );
	printf ( "                                                                              \n" );

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Th-th-th-that's all, folks!                                                           | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	exit ( exit_code );
	
	}



/* ******************************************************************************************** */
/* *                                                                                          * */
/* *    This routine open the Library                                                         * */
/* *                                                                                          * */
/* ******************************************************************************************** */

static OSErr library_open (  )
	
	{
	OSErr		the_result;								/* Holds error returns from routines	*/
	OSErr		exit_code;								/* Holds error returns from routines	*/
	
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Initialize the X.25 Interface Library                                                 | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	the_result = X25_Open ( nil, application_type );

	if ( the_result != X25if_no_error )
		{
		printf ( "### X25 Library Initialization Failed: return_code = 0x%04X \n", the_result );
		exit (1);
		}
	
	printf ( "#   X25 Library Opened \n" );
	fflush ( stdout );

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |	Now connected to the X.25 Server.  Go to a subroutine to make an X.25 SVC connection  | */
/* |    and exchange some data.                                                               | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	exit_code = vccb_open ( );
	

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Close the X.25 Library.                                                               | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	the_result = X25_Close ( );

	if ( the_result != X25if_no_error )
		{
		printf ( "### X25 Library Close Failed: return_code = 0x%04X \n", the_result );
		exit (1);
		}
	
	printf ( "#   X25 Library Closed \n" );
	fflush ( stdout );

	
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    All done, return.                                                                     | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		return ( exit_code );
	}



/* ******************************************************************************************** */
/* *                                                                                          * */
/* *    This routine first initializes our "tcb" (Transmission Control Block). It then        * */
/* *    makes the Library call to initialize the "vccb" (Virtual Circuit Control Block),      * */
/* *    and then calls the next level of processing.  On return, it closes the "vccb".        * */
/* *                                                                                          * */
/* ******************************************************************************************** */

static OSErr vccb_open (  )
	
	{
	OSErr		the_result;								/* Holds error returns from routines	*/
	OSErr		exit_code;								/* Holds error returns from routines	*/
	
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Clear the tcb area before use                                                         | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	{
		register	char	* 	to_ptr	= (char*)tcb_ptr;
		register	int			count	= sizeof ( tcb_def );
		
		while ( count-- > 0 )
			*to_ptr++ = 0;
	}


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Initialize the control block for a new circuit.                                       | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |    Note: The function return value and the value stored in tcb_ptr -> return_code        | */
/* |          are identical in all cases EXCEPT the case where the address of aCircuit        | */
/* |          is NULL. If you are confident that your code cannot make this error, you        | */
/* |          need only check the tcb_ptr -> return_code value.                               | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	the_result = X25_vccb_Open ( tcb_ptr, nil, MY_BUF_SZ );

	if ( the_result != X25if_no_error )
		{
		error_print ( tcb_ptr, "vccb Initialization Failed:" );
		return ( 1 );
		}
	
	printf ( "#   vccb Opened \n" );
	fflush ( stdout );

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |	Now go to the next level of processing which will open a connection to the            | */
/* |    X.25 Server.                                                                          | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	exit_code = server_open ( );


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Close the vccb.                                                                       | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	the_result = X25_vccb_Close ( tcb_ptr );

	if ( the_result != X25if_no_error )
		{
		printf ( "vccb Close Failed: return_code = 0x%04X \n", the_result );
		return ( 1 );
		}
	
	printf ( "#   vccb Closed \n" );
	fflush ( stdout );

	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    All done, return.                                                                     | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		return ( exit_code );
	}



/* ******************************************************************************************** */
/* *                                                                                          * */
/* *    This routine handles the connection to the specified X.25 Server.  Once connection    * */
/* *    is established, another subroutine is called for the next level of processing.        * */
/* *                                                                                          * */
/* ******************************************************************************************** */

static OSErr server_open (  )
	
	{
	OSErr		the_result;						/* Holds error returns from routines			*/
	OSErr		exit_code;						/* Holds returns from next lower level routines	*/
	
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Obtains the server's AppleTalk Name and Zone as last selected with the Chooser.       | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	the_result = X25_Chooser ( tcb_ptr );

	if ( the_result != X25if_no_error )
		{
		error_print ( tcb_ptr, "Access to Chooser Failed:" );
		return ( 1 );
		}
	
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Get the Server's Name and Zone and print this information.                            | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	{
		char		server_name [256]	= "\p";
		char		server_zone [256]	= "\p";
	
		the_result = X25_Get_Server_ID ( tcb_ptr, server_name, server_zone );
	
		if ( the_result != X25if_no_error )
			{
			error_print ( tcb_ptr, "Retreival of Server Name/Zone Failed:" );
			return ( 1 );
			}
		
		printf ( "#   Server Name:                        = '%P' \n", server_name );
		printf ( "#   Server Zone:                        = '%P' \n", server_zone );
		fflush ( stdout );
	}
	
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Setup the tcb for the server connection call.                                         | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		tcb_ptr -> aux_write_data.data_ptr		= aux_write_buf;
		tcb_ptr -> aux_write_data.max_length	= sizeof ( aux_write_buf );

		tcb_ptr -> write_data.data_ptr			= write_buf;
		tcb_ptr -> write_data.max_length		= sizeof ( write_buf );

		tcb_ptr -> read_data.data_ptr			= read_buf;
		tcb_ptr -> read_data.req_length			= sizeof ( read_buf );
		tcb_ptr -> read_data.max_length			= sizeof ( read_buf );

		tcb_ptr -> aux_read_data.data_ptr		= aux_read_buf;
		tcb_ptr -> aux_read_data.req_length		= sizeof ( aux_read_buf );
		tcb_ptr -> aux_read_data.max_length		= sizeof ( aux_read_buf );

		tcb_ptr -> write_data.IF_asynchronous	= false;
		tcb_ptr -> write_data.completion		= nil;
		tcb_ptr -> write_data.timeout			= timeout_sec ( 5 );

		tcb_ptr -> read_data.IF_asynchronous	= false;
		tcb_ptr -> read_data.completion			= nil;
		tcb_ptr -> read_data.timeout			= timeout_sec ( 5 );

		tcb_ptr -> ref_con						= nil;

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Connect to the specified X.25 Server.                                                 | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	the_result = X25_Server_Connect ( tcb_ptr, connect_via_best );

	if ( the_result != X25if_no_error )
		{
		error_print ( tcb_ptr, "Connection to Server Failed:" );
		return ( 1 );
		}
	
	printf ( "#   Server Connection Opened \n" );
	fflush ( stdout );
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |	Now connected to the X.25 Server.  Go to a subroutine to make an X.25 SVC connection  | */
/* |    and exchange some data.                                                               | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	exit_code = user_validate ( );
	

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |	All done, need to disconnect from the X.25 Server.                                    | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |    Note: A major problem during development is exiting the program without closing       | */
/* |          the connection to the server.  This means that long AFTER you exit, data        | */
/* |          may arrive and be placed in a buffer that WAS in your application (stack,       | */
/* |          heap, or elsewhere).  Needless to say, a new application that may be running    | */
/* |          is usually not equipped to handle enexpected data arriving at an enexpected     | */
/* |          address, and will probably crash.                                               | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */


	the_result = X25_Server_Discnct ( tcb_ptr );

	if ( the_result != X25if_no_error )
		{
		error_print ( tcb_ptr, "Disconnection from Server Failed:" );
		return ( 1 );
		}
	
	printf ( "#   Server Connection Closed \n" );
	fflush ( stdout );
	
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    All done, return.                                                                     | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		return ( exit_code );
	}



/* ******************************************************************************************** */
/* *                                                                                          * */
/* *    This routine validates that the user is allowed to use the specified X.25 Server.     * */
/* *                                                                                          * */
/* ******************************************************************************************** */

static OSErr user_validate (  )
	
	{
		OSErr		the_result;							/* Holds error returns from routines	*/
		OSErr		exit_code;							/* Holds error returns from routines	*/
	
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Set the supplied user name and password                                               | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		printf ( "#   User Name:                          = '%P' \n", user_name_ptr     );
		printf ( "#   User Password:                      = '%P' \n", user_password_ptr );
	
		the_result = X25_Set_User_ID ( tcb_ptr, user_name_ptr     ,
		                                        user_password_ptr );
	
		if ( the_result != X25if_no_error )
			{
			error_print ( tcb_ptr, "Could Not Set User ID:" );
			return ( 1 );
			}


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Setup the tcb for the validation call.                                                | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		tcb_ptr -> aux_write_data.data_ptr		= aux_write_buf;
		tcb_ptr -> aux_write_data.max_length	= sizeof ( aux_write_buf );

		tcb_ptr -> write_data.data_ptr			= write_buf;
		tcb_ptr -> write_data.max_length		= sizeof ( write_buf );

		tcb_ptr -> read_data.data_ptr			= read_buf;
		tcb_ptr -> read_data.req_length			= sizeof ( read_buf );
		tcb_ptr -> read_data.max_length			= sizeof ( read_buf );

		tcb_ptr -> aux_read_data.data_ptr		= aux_read_buf;
		tcb_ptr -> aux_read_data.req_length		= sizeof ( aux_read_buf );
		tcb_ptr -> aux_read_data.max_length		= sizeof ( aux_read_buf );

		tcb_ptr -> write_data.IF_asynchronous	= false;
		tcb_ptr -> write_data.completion		= nil;
		tcb_ptr -> write_data.timeout			= timeout_sec ( 5 );

		tcb_ptr -> read_data.IF_asynchronous	= false;
		tcb_ptr -> read_data.completion			= nil;
		tcb_ptr -> read_data.timeout			= timeout_sec ( 5 );

		tcb_ptr -> ref_con						= nil;

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Make the validation call.                                                             | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	the_result = X25_Server_Authorize ( tcb_ptr );

	if ( the_result != X25if_no_error )
		{
		error_print ( tcb_ptr, "Authorization to Server Failed:" );
		return ( 1 );
		}
	
	printf ( "#   Server Authorization Made \n" );
	fflush ( stdout );
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |	Now connected to the X.25 Server.  Go to a subroutine to make an X.25 SVC connection  | */
/* |    and exchange some data.                                                               | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	exit_code = x25_open ( );
	

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    All done, return.                                                                     | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		return ( exit_code );
	}



/* ******************************************************************************************** */
/* *                                                                                          * */
/* *	x25_open		                                                                      * */
/* *                                                                                          * */
/* *------------------------------------------------------------------------------------------* */
/* *                                                                                          * */
/* *    This routine (attempts to) establishes an X.25 SVC to the selected address.  After    * */
/* *    the connection is made, it calls another routine to ship data in several different    * */
/* *    modes.  On return, it closes the X.25 SVC and returns.                                * */
/* *                                                                                          * */
/* ******************************************************************************************** */

static OSErr	x25_open       	(  )
	
	{

		OSErr		the_result;							/* Holds error returns from routines	*/
		OSErr		exit_code;							/* Holds error returns from routines	*/


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Print the X.25 Connection Record.                                                     | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		printf ( "#   Connection Record:                  = '%P' \n", connect_rec_ptr );
		
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Setup the tcb for the "X25_Connect" call.                                             | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	"aux_write_data.data_ptr" is set to point to the supplied Connection          | */
/* |            Record.                                                                       | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		tcb_ptr -> aux_write_data.data_ptr		= connect_rec_ptr;

		tcb_ptr -> write_data.data_ptr			= write_buf;
		tcb_ptr -> write_data.max_length		= sizeof ( write_buf );

		tcb_ptr -> read_data.data_ptr			= read_buf;
		tcb_ptr -> read_data.req_length			= sizeof ( read_buf );
		tcb_ptr -> read_data.max_length			= sizeof ( read_buf );

		tcb_ptr -> aux_read_data.data_ptr		= aux_read_buf;
		tcb_ptr -> aux_read_data.req_length		= sizeof ( aux_read_buf );
		tcb_ptr -> aux_read_data.max_length		= sizeof ( aux_read_buf );

		tcb_ptr -> write_data.IF_asynchronous	= false;
		tcb_ptr -> write_data.completion		= nil;
		tcb_ptr -> write_data.timeout			= timeout_sec ( 5 );

		tcb_ptr -> read_data.IF_asynchronous	= false;
		tcb_ptr -> read_data.completion			= nil;
		tcb_ptr -> read_data.timeout			= timeout_sec ( 5 );

		tcb_ptr -> ref_con						= nil;

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |	Make the X.25 SVC Connection.                                                         | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	X.25_Connect will take an ASCII text string as the call placement record,     | */
/* |            build a call request packet and send it, then read the returned packet        | */
/* |            to see if it was a call accepted packet or a clear packet (or something       | */
/* |            else).                                                                        | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		the_result = X25_Connect ( tcb_ptr );

		if ( the_result == X25if_call_cleared )
			{
			printf ( "# \n" );
			printf ( "### %P \n", tcb_ptr -> aux_read_data.data_ptr );
			}
		
		if ( the_result != X25if_no_error )
			{
			error_print ( tcb_ptr, "X25 SVC Connection Failed:" );
			return ( 1 );
			}
		
		printf ( "#   X.25 SVC Connection Opened \n" );
		fflush ( stdout );
	
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |	Now have an X.25 SVC Connection.  Go to another subroutine which will exercise the    | */
/* |    path by exchanging data with the remote end.                                          | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		exit_code = data_test ( );

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Setup the tcb for the "X25_Discnct" call.                                             | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		tcb_ptr -> aux_write_data.data_ptr		= aux_write_buf;
		tcb_ptr -> aux_write_data.max_length	= sizeof ( aux_write_buf );

		tcb_ptr -> write_data.data_ptr			= write_buf;
		tcb_ptr -> write_data.max_length		= sizeof ( write_buf );

		tcb_ptr -> read_data.data_ptr			= read_buf;
		tcb_ptr -> read_data.req_length			= sizeof ( read_buf );
		tcb_ptr -> read_data.max_length			= sizeof ( read_buf );

		tcb_ptr -> aux_read_data.data_ptr		= aux_read_buf;
		tcb_ptr -> aux_read_data.req_length		= sizeof ( aux_read_buf );
		tcb_ptr -> aux_read_data.max_length		= sizeof ( aux_read_buf );

		tcb_ptr -> write_data.IF_asynchronous	= false;
		tcb_ptr -> write_data.completion		= nil;
		tcb_ptr -> write_data.timeout			= timeout_sec ( 5 );

		tcb_ptr -> read_data.IF_asynchronous	= false;
		tcb_ptr -> read_data.completion			= nil;
		tcb_ptr -> read_data.timeout			= timeout_sec ( 5 );

		tcb_ptr -> ref_con						= nil;
		
		
		aux_write_buf[0] = '\0';				/* Force a zero length string					*/


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |	Clear the X.25 SVC Connection.                                                        | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	This isn't really necessary.  When you do the Server Disconnect, the          | */
/* |            Server Agent will notify the X.25 code who will then ensure that the          | */
/* |            X.25 SVC is closed.  However, its better to do things by the book in          | */
/* |            case of future changes to the software.                                       | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		the_result = X25_Discnct ( tcb_ptr );
	
		if ( the_result != X25if_no_error )
			{
			error_print ( tcb_ptr, "X25 SVC Disconnection Failed:" );
			return ( 1 );
			}
	
		printf ( "#   X.25 SVC Connection Closed \n" );
		fflush ( stdout );
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    All done, return.                                                                     | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		return ( exit_code );
	}




/* ******************************************************************************************** */
/* *                                                                                          * */
/* *	data_test                                                                             * */
/* *                                                                                          * */
/* *------------------------------------------------------------------------------------------* */
/* *                                                                                          * */
/* *    At this point we have an X.25 SVC to a remote entity.  We may now make reads and      * */
/* *    writes of arbitrary data to our heart's content.  This routine will several           * */
/* *    different routines to send and receive data to demonstrate the various modes of       * */
/* *    I/O operation.                                                                        * */
/* *                                                                                          * */
/* *    For all these examples, we assume that the remote entity is an "echo process", which  * */
/* *    will simply return each of our packets to us. Each write will be followed by a        * */
/* *    read, and in these particular circumstances, we can consider it an error if we        * */
/* *    don't get back exactly what we sent.                                                  * */
/* *                                                                                          * */
/* ******************************************************************************************** */

static OSErr	data_test       	(  )
	
	{
		OSErr		exit_code;							/* Holds error returns from routines	*/


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Synchronous, blocking call with timeouts                                              | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		{
			int		count	= 1;
			
			while ( count-- > 0 )
				exit_code = data_test_1 ( );
		}
	

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Synchronous, blocking call with software polling loop                                 | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		{
			int		count	= 2;
			
			while ( count-- > 0 )
				exit_code = data_test_2 ( );
		}
	

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Asynchronous call with software polling loop                                          | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		{
			int		count	= 1;
			
			while ( count-- > 0 )
				exit_code = data_test_3 ( );
		}
	

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    All Done, Return                                                                      | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		return ( 0 );
	}

	

/* ******************************************************************************************** */
/* *                                                                                          * */
/* *	data_test_1		( synchronous, blocking, timeouts )                                   * */
/* *                                                                                          * */
/* *------------------------------------------------------------------------------------------* */
/* *                                                                                          * */
/* *    This test uses a "synchronous", "blocking" write, withe "timeouts", followed by the   * */
/* *    same type of read".                                                                   * */
/* *                                                                                          * */
/* *    A "Non-blocking write means if the circuit is flow-controlled in the outbound         * */
/* *    direction, the X25_Write call will return WITHOUT ERROR but with ZERO BYTES WRITTEN.  * */
/* *    In this example, it is highly unlikely that a write will be flow-controlled           * */
/* *    (especially the first write after a call connection), but use your imagination.       * */
/* *                                                                                          * */
/* ******************************************************************************************** */

static 	OSErr		data_test_1       	(  )
	
	{

		OSErr		the_result;							/* Holds error returns from routines	*/
		
		char		xmit_data [256]	= "\pHello World!";	/* The data that we are to send			*/
		char		recv_data [256];	                /* The data that we get back			*/


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Setup the tcb for the "X25_encode_data" call, then make the encode call.              | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	The first line of code sets the pointer of the data to be encoded to be       | */
/* |            the address of the Pascal string.  The second line of code sets the length    | */
/* |            to be that of the string (string length byte plus 1).                         | */
/* |                                                                                          | */
/* |            The second two lines of code set the buffer where the encoded packet is       | */
/* |            to be built.  (These values are already set, but for this example we do       | */
/* |            it again.)                                                                    | */
/* |                                                                                          | */
/* |            The encode routine will set the "write_data.req_length" to the correct        | */
/* |            value for the X25_Write call.                                                 | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
		
		tcb_ptr -> aux_write_data.data_ptr     	= &(xmit_data[0]);	
		tcb_ptr -> aux_write_data.req_length    = xmit_data[0] + 1;
		
		tcb_ptr -> write_data.data_ptr 			= write_buf;
		tcb_ptr -> write_data.max_length 		= sizeof ( write_buf );

		the_result = X25_encode_data ( tcb_ptr );
	
		if ( the_result != X25if_no_error )
			{
			error_print ( tcb_ptr, "Data_Test_1: Encode of Data Failed:" );
			return ( 1 );
			}
	
		printf ( "#   Data_Test_1: Data Encode Completed \n" );
		fflush ( stdout );
		
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Setup the tcb for the "X25_Write" call.                                               | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	The "write_buffer.data_ptr" was set for the encode call and is still intact.  | */
/* |            "write_buffer.req_length" was set by the encode call.                         | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		tcb_ptr -> write_data.timeout    		= timeout_sec ( 5 );
		tcb_ptr -> write_data.IF_asynchronous 	= false;
		tcb_ptr -> write_data.IF_more         	= false;
		tcb_ptr -> write_data.IF_expedited    	= false;
	
		tcb_ptr -> write_data.completion    	= nil;		/* not used on synchronous calls	*/
	

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Make the "X25_Write" call.                                                            | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	We loop until either the data is written, the write call fails, or until      | */
/* |            the number of attempts exceeds a limit (a poor man's timeout).  In real       | */
/* |            life, we'd find something useful to do with our time in between polling       | */
/* |            the X25_Write.                                                                | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
		
		the_result = X25_Write ( tcb_ptr );
		
		if ( the_result != X25if_no_error )
			{
			error_print ( tcb_ptr, "Data_Test_1 X.25 Write Failed:" );
			X25_Server_Discnct (tcb_ptr);
			return ( 1 );
			}
	
		printf ( "#   Data_Test_1: X.25 Write Completed \n" );
		fflush ( stdout );
	

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Setup the tcb for the "X25_Read" call.                                                | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		tcb_ptr -> read_data.timeout    		= timeout_sec ( 5 );
		tcb_ptr -> read_data.IF_asynchronous 	= false;
	
		tcb_ptr -> read_data.completion    		= nil;
	
		tcb_ptr -> read_data.data_ptr 			= read_buf;
		tcb_ptr -> read_data.max_length 		= sizeof ( read_buf );


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Make the "X25_Read" call.                                                             | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	Like the previous Write call, this is a synchronous, non-blocking Read.       | */
/* |            Non-blocking reads return WITHOUT ERROR but WITH ZERO BYTES READ if there     | */
/* |            is no complete packet to read, so again we loop until error or success.       | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
		
		the_result = X25_Read ( tcb_ptr );
		
		if ( the_result != X25if_no_error )
			{
			error_print ( tcb_ptr, "Data_Test_1 X.25 Read Failed:" );
			return ( 1 );
			}
	
		printf ( "#   Data_Test_1: X.25 Read Completed \n" );
		fflush ( stdout );
	

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Setup the tcb for the "X25_decode_data" call, then make the decode call.              | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	The first line of code sets the pointer as to where the data is to be         | */
/* |            stored.  The second line of code is the max size of this buffer, and the      | */
/* |            third line is how much data we actually want to receive.                      | */
/* |                                                                                          | */
/* |            "read_data.data_ptr" is still set from the write call, and                    | */
/* |            "read_data.act_length" was set by X25_Read.                                   | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
		
		tcb_ptr -> aux_read_data.data_ptr     	= &recv_data[0];	
		tcb_ptr -> aux_read_data.max_length    	= sizeof ( recv_data );
		tcb_ptr -> aux_read_data.req_length    	= sizeof ( recv_data );
		
		the_result = X25_decode_data ( tcb_ptr );
	
		if ( the_result != X25if_no_error )
			{
			error_print ( tcb_ptr, "Data_Test_1: Decode of Data Failed:" );
			return ( 1 );
			}
	
		printf ( "#   Data_Test_1: Data Decode Completed \n" );
		fflush ( stdout );
		
		printf ( "#   Data_Test_1: Received Data:         = '%P' \n", tcb_ptr -> aux_read_data.data_ptr );
		fflush ( stdout );
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    All done, return.                                                                     | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		return ( 0 );
	}


/* ******************************************************************************************** */
/* *                                                                                          * */
/* *	data_test_2		( synchronous, blocking, external timeout loop )                      * */
/* *                                                                                          * */
/* *------------------------------------------------------------------------------------------* */
/* *                                                                                          * */
/* *    This test will make a "synchronous, non-blocking write", followed by a                * */
/* *    "synchronous, non-blocking read".                                                     * */
/* *                                                                                          * */
/* *    A "Non-blocking write means if the circuit is flow-controlled in the outbound         * */
/* *    direction, the X25_Write call will return WITHOUT ERROR but with ZERO BYTES WRITTEN.  * */
/* *    In this example, it is highly unlikely that a write will be flow-controlled           * */
/* *    (especially the first write after a call connection), but use your imagination.       * */
/* *                                                                                          * */
/* ******************************************************************************************** */

static 	OSErr		data_test_2       	(  )
	
	{

		OSErr		the_result;							/* Holds error returns from routines	*/
		
		long		attempt_count;						/* Attempt counter for Reads / Writes	*/
		
		char		xmit_data [256]	= "\pNice Day";		/* The data that we are to send			*/
		char		recv_data [256];	                /* The data that we get back			*/


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Setup the tcb for the "X25_encode_data" call, then make the encode call.              | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	The first line of code sets the pointer of the data to be encoded to be       | */
/* |            the address of the Pascal string.  The second line of code sets the length    | */
/* |            to be that of the string (string length byte plus 1).                         | */
/* |                                                                                          | */
/* |            The second two lines of code set the buffer where the encoded packet is       | */
/* |            to be built.  (These values are already set, but for this example we do       | */
/* |            it again.)                                                                    | */
/* |                                                                                          | */
/* |            The encode routine will set the "write_data.req_length" to the correct        | */
/* |            value for the X25_Write call.                                                 | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
		
		tcb_ptr -> aux_write_data.data_ptr     	= &xmit_data[0];	
		tcb_ptr -> aux_write_data.req_length    = xmit_data[1] + 1;
		
		tcb_ptr -> write_data.data_ptr 			= write_buf;
		tcb_ptr -> write_data.max_length 		= sizeof ( write_buf );

		the_result = X25_encode_data ( tcb_ptr );
	
		if ( the_result != X25if_no_error )
			{
			error_print ( tcb_ptr, "Data_Test_2: Encode of Data Failed:" );
			return ( 1 );
			}
	
		printf ( "#   Data_Test_2: Data Encode Completed \n" );
		fflush ( stdout );
		
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Setup the tcb for the "X25_Write" call.                                               | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	The "write_buffer.data_ptr" was set for the encode call and is still intact.  | */
/* |            "write_buffer.req_length" was set by the encode call.                         | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		tcb_ptr -> write_data.IF_asynchronous 	= false;
		tcb_ptr -> write_data.IF_more         	= false;
		tcb_ptr -> write_data.IF_expedited    	= false;
	
		tcb_ptr -> write_data.completion    	= nil;		/* not used on synchronous calls	*/
		tcb_ptr -> write_data.timeout    		= 0;
	

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Make the "X25_Write" call.                                                            | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	We loop until either the data is written, the write call fails, or until      | */
/* |            the number of attempts exceeds a limit (a poor man's timeout).  In real       | */
/* |            life, we'd find something useful to do with our time in between polling       | */
/* |            the X25_Write.                                                                | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
		
		attempt_count = 10000;
		
		do
			{
			
			if ( attempt_count-- <= 0 )
				{
				error_print ( tcb_ptr, "Sync X.25 Write Timeout:" );
				return ( 1 );
				}
	
			the_result = X25_Write ( tcb_ptr );
			
			if ( the_result == X25if_io_would_block )
				continue;

			if ( the_result != X25if_no_error )
				{
				error_print ( tcb_ptr, "Data_Test_2: X.25 Write Failed:" );
				X25_Server_Discnct (tcb_ptr);
				return ( 1 );
				}
	
			} while ( tcb_ptr -> write_data.act_length == 0 );
		
		printf ( "#   Data_Test_2: X.25 Write Completed \n" );
		fflush ( stdout );
	

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Setup the tcb for the "X25_Read" call.                                                | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		tcb_ptr -> read_data.IF_asynchronous 	= false;
	
		tcb_ptr -> read_data.completion    		= nil;
		tcb_ptr -> read_data.timeout    		= 0;
	
		tcb_ptr -> read_data.data_ptr 			= read_buf;
		tcb_ptr -> read_data.max_length 		= sizeof ( read_buf );


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Make the "X25_Read" call.                                                             | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	Like the previous Write call, this is a synchronous, non-blocking Read.       | */
/* |            Non-blocking reads return WITHOUT ERROR but WITH ZERO BYTES READ if there     | */
/* |            is no complete packet to read, so again we loop until error or success.       | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
		
		attempt_count = 10000;
		
		do
			{

			if ( attempt_count-- <= 0 )
				{
				error_print ( tcb_ptr, "Data_Test_2: X.25 Read Timeout:" );
				return ( 1 );
				}
	
			the_result = X25_Read ( tcb_ptr );
			
			if ( the_result == X25if_io_would_block )
				continue;

			if ( the_result != X25if_no_error )
				{
				error_print ( tcb_ptr, "Sync X.25 Read Failed:" );
				return ( 1 );
				}
	
			} while ( tcb_ptr -> read_data.act_length == 0 );
		
		printf ( "#   Data_Test_2: X.25 Read Completed \n" );
		fflush ( stdout );
	

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Setup the tcb for the "X25_decode_data" call, then make the decode call.              | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	The first line of code sets the pointer as to where the data is to be         | */
/* |            stored.  The second line of code is the max size of this buffer, and the      | */
/* |            third line is how much data we actually want to receive.                      | */
/* |                                                                                          | */
/* |            "read_data.data_ptr" is still set from the write call, and                    | */
/* |            "read_data.act_length" was set by X25_Read.                                   | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
		
		tcb_ptr -> aux_read_data.data_ptr     	= &recv_data[0];	
		tcb_ptr -> aux_read_data.max_length    	= sizeof ( recv_data );
		tcb_ptr -> aux_read_data.req_length    	= sizeof ( recv_data );
		
		the_result = X25_decode_data ( tcb_ptr );
	
		if ( the_result != X25if_no_error )
			{
			error_print ( tcb_ptr, "Data_Test_2: Decode of Data Failed:" );
			return ( 1 );
			}
	
		printf ( "#   Data_Test_2: Data Decode Completed \n" );
		fflush ( stdout );
		
		printf ( "#   Data_Test_2: Received Data:         = '%P' \n", tcb_ptr -> aux_read_data.data_ptr );
		fflush ( stdout );

	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    All done, return.                                                                     | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		return ( 0 );
	}




/* ******************************************************************************************** */
/* *                                                                                          * */
/* *	data_test_3		( asynchronous, external timeout loop )                               * */
/* *                                                                                          * */
/* *------------------------------------------------------------------------------------------* */
/* *                                                                                          * */
/* *    This test will make an "asynchronous" write, followed by an "asynchronous" read.      * */
/* *    An external polling loop will detect timeouts.                                        * */
/* *                                                                                          * */
/* *    A "Non-blocking write means if the circuit is flow-controlled in the outbound         * */
/* *    direction, the X25_Write call will return WITHOUT ERROR but with ZERO BYTES WRITTEN.  * */
/* *    In this example, it is highly unlikely that a write will be flow-controlled           * */
/* *    (especially the first write after a call connection), but use your imagination.       * */
/* *                                                                                          * */
/* ******************************************************************************************** */

static 	OSErr		data_test_3       	(  )
	
	{

		OSErr		the_result;							/* Holds error returns from routines	*/
		
		long		attempt_count;						/* Attempt counter for Reads / Writes	*/
		
		char		xmit_data [256]	= "\pEnjoy It !";	/* The data that we are to send			*/
		char		recv_data [256];	                /* The data that we get back			*/


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Setup the tcb for the "X25_encode_data" call, then make the encode call.              | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	The first line of code sets the pointer of the data to be encoded to be       | */
/* |            the address of the Pascal string.  The second line of code sets the length    | */
/* |            to be that of the string (string length byte plus 1).                         | */
/* |                                                                                          | */
/* |            The second two lines of code set the buffer where the encoded packet is       | */
/* |            to be built.  (These values are already set, but for this example we do       | */
/* |            it again.)                                                                    | */
/* |                                                                                          | */
/* |            The encode routine will set the "write_data.req_length" to the correct        | */
/* |            value for the X25_Write call.                                                 | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
		
		tcb_ptr -> aux_write_data.data_ptr     	= &xmit_data[0];	
		tcb_ptr -> aux_write_data.req_length    = xmit_data[1] + 1;
		
		tcb_ptr -> write_data.data_ptr 			= write_buf;
		tcb_ptr -> write_data.max_length 		= sizeof ( write_buf );

		the_result = X25_encode_data ( tcb_ptr );
	
		if ( the_result != X25if_no_error )
			{
			error_print ( tcb_ptr, "Data_Test_3: Encode of Data Failed:" );
			return ( 1 );
			}
	
		printf ( "#   Data_Test_3: Data Encode Completed \n" );
		fflush ( stdout );
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Setup the tcb for the "X25_Write" call.                                               | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	The "write_buffer.data_ptr" was set for the encode call and is still intact.  | */
/* |            "write_buffer.req_length" was set by the encode call.                         | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		tcb_ptr -> write_data.IF_asynchronous 	= true;
		tcb_ptr -> write_data.IF_more         	= false;
		tcb_ptr -> write_data.IF_expedited    	= false;
	
		tcb_ptr -> write_data.completion    	= nil;		/* we won't use on this call		*/
		tcb_ptr -> write_data.timeout    		= 0;
	

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Make the "X25_Write" call.                                                            | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	We get immediate status as usual, but this only indicates that the data       | */
/* |            did (or didn't) successfully get queued for later execution.                  | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
		
		/*	Initiate the Write Operation 														*/ 
		
		the_result = X25_Write ( tcb_ptr );
		
		if (  ( the_result != X25if_io_pending )
		   && ( the_result != X25if_no_error   ) )
			{
			error_print ( tcb_ptr, "Data_Test_3: X.25 Write Schedule Failed:" );
			X25_Server_Discnct (tcb_ptr);
			return ( 1 );
			}

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Wait for the Write Operation to Complete.                                             | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	The async call to X25_Write sets write_data.return_code to "X25_io_pending".  | */
/* |            When the call completes, its value will change.  We loop until that happens.  | */
/* |            In real life, we'd find something useful to do with our time in between       | */
/* |            polling write_data.status_code.                                               | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
		
		attempt_count = 1000000;
		
		do
			{
			
			if ( attempt_count-- <= 0 )
				{
				X25_Kill_Write ( tcb_ptr );
				error_print ( tcb_ptr, "Sync X.25 Write Timeout:" );
				return ( 1 );
				}
	
			} while ( tcb_ptr -> write_data.return_code == X25if_io_pending );
		
		if ( tcb_ptr -> write_data.return_code != X25if_no_error )
			{
			tcb_ptr -> return_code = tcb_ptr -> write_data.return_code;
			tcb_ptr -> status_code = tcb_ptr -> write_data.status_code;
			error_print ( tcb_ptr, "Data_Test_3: X.25 Write Completion Failed:" );
			X25_Server_Discnct (tcb_ptr);
			return ( 1 );
			}

		printf ( "#   Data_Test_3: X.25 Write Completed \n" );
		fflush ( stdout );
	

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Setup the tcb for the "X25_Read" call.                                                | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		tcb_ptr -> read_data.IF_asynchronous 	= true;
	
		tcb_ptr -> read_data.completion    		= nil;
		tcb_ptr -> read_data.timeout    		= 0;
	
		tcb_ptr -> read_data.data_ptr 			= read_buf;
		tcb_ptr -> read_data.max_length 		= sizeof ( read_buf );


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Make the "X25_Read" call.                                                             | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	Same comments as for the Read call.                                           | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
		
		/*	Initiate the Read Operation 														*/ 
		
		the_result = X25_Read ( tcb_ptr );
		
		if (  ( the_result != X25if_io_pending )
		   && ( the_result != X25if_no_error   ) )
			{
			error_print ( tcb_ptr, "Data_Test_3: X.25 Read Schedule Failed:" );
			X25_Server_Discnct (tcb_ptr);
			return ( 1 );
			}

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Wait for the Read Operation to Complete.                                              | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	Same comments as for the Read call.                                           | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
		
		attempt_count = 1000000;
		
		do
			{

			if ( attempt_count-- <= 0 )
				{
				X25_Kill_Read ( tcb_ptr );
				error_print ( tcb_ptr, "Data_Test_3: X.25 Read Timeout:" );
				return ( 1 );
				}
	
			} while ( tcb_ptr -> read_data.return_code == X25if_io_pending );
		
		if ( tcb_ptr -> read_data.return_code != X25if_no_error )
			{
			tcb_ptr -> return_code = tcb_ptr -> read_data.return_code;
			tcb_ptr -> status_code = tcb_ptr -> read_data.status_code;
			error_print ( tcb_ptr, "Data_Test_3: X.25 Read Completion Failed:" );
			X25_Server_Discnct (tcb_ptr);
			return ( 1 );
			}

	
		printf ( "#   Data_Test_3: X.25 Read Completed \n" );
		fflush ( stdout );
	

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Setup the tcb for the "X25_decode_data" call, then make the decode call.              | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	The first line of code sets the pointer as to where the data is to be         | */
/* |            stored.  The second line of code is the max size of this buffer, and the      | */
/* |            third line is how much data we actually want to receive.                      | */
/* |                                                                                          | */
/* |            "read_data.data_ptr" is still set from the write call, and                    | */
/* |            "read_data.act_length" was set by X25_Read.                                   | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
		
		tcb_ptr -> aux_read_data.data_ptr     	= &recv_data[0];	
		tcb_ptr -> aux_read_data.max_length    	= sizeof ( recv_data );
		tcb_ptr -> aux_read_data.req_length    	= sizeof ( recv_data );
		
		the_result = X25_decode_data ( tcb_ptr );
	
		if ( the_result != X25if_no_error )
			{
			error_print ( tcb_ptr, "Data_Test_3: Decode of Data Failed:" );
			return ( 1 );
			}
	
		printf ( "#   Data_Test_3: Data Decode Completed \n" );
		fflush ( stdout );
		
		printf ( "#   Data_Test_3: Received Data:         = '%P' \n", tcb_ptr -> aux_read_data.data_ptr );
		fflush ( stdout );

	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    All done, return.                                                                     | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		return ( 0 );
	}



/* ******************************************************************************************** */
/* *                                                                                          * */
/* *    Format and printout any errors that occur                                             * */
/* *                                                                                          * */
/* ******************************************************************************************** */

static void	error_print       	( tcb_def 			* 	const	tcb_ptr			, 
								  char        		*  	const	message  		)
	
	{
		OSErr						the_result;
		char						msg_area [256]	= "\p";

		printf ( "# \n" );

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Do a simple printout of where the failure occured along with the return and status    | */
/* |    codes.                                                                                | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		printf ( "### %s return_code = 0x%04X, status_code = 0x%04X\n" , 
				 message                                               ,
		         tcb_ptr -> return_code & 0x0000FFFF                   ,
		         tcb_ptr -> status_code & 0x0000FFFF                   );
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Call the Library routine to generate an decoded message and print this information.   | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		the_result = X25_error_decode ( tcb_ptr -> return_code , 
		                                tcb_ptr -> status_code , 
										msg_area               );
		
		printf ( "### %P \n", msg_area );
	

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    All done, return.                                                                     | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		printf ( "# \n" );
		fflush ( stdout );

		return;
	}
	