/* ******************************************************************************************** */
/* *                                                                     		              * */
/* *                              X.25 Interface Library                                      * */
/* *                              ----------------------                                      * */
/* *                                                                     		              * */
/* *                                   Example_2.c                                            * */
/* *                                                                     		              * */
/* *..........................................................................................* */
/* *                                                                                          * */
/* *     Example code for developers.                                                         * */
/* *                                                                                          * */
/* *     This program will register to receive an incoming call with "OBF" (Quick Brown Fox)  * */
/* *     in the Call User Data Field.  On receipt of a call, it will send out 9 lines         * */
/* *     (one line per packet) of:                                                            * */
/* *                                                                                          * */
/* *         "nn The quick brown fox jumped over the lazy dog's back. \n"                     * */
/* *                                                                                          * */
/* *     where nn is the line number.  After sending the last packet, it will send an         * */
/* *     "invitation to clear" (pad control message), wait 20 seconds, and send a clear       * */
/* *     packet and exit.                                                                     * */
/* *                                                                                          * */
/* *------------------------------------------------------------------------------------------* */
/* *                                                                     		              * */
/* *                   Copyright  1988-1991 Apple Computer, Inc.              		          * */
/* *                               All Rights Reserved                        		          * */
/* *                           Apple Computer Confidential                     		          * */
/* *                                                                     		              * */
/* *------------------------------------------------------------------------------------------* */
/* *                                                                                          * */
/* *     Revision History:                                                                    * */
/* *                                                                                          * */
/* *                                                                                          * */
/* *     Date       Individual          Changes                                               * */
/* *     .........  ................... ..................................................    * */
/* *                                                                     		              * */
/* *     03 Dec 89  D. N. Willey        Original version.                                     * */
/* *                                                                                          * */
/* *     16 Apr 91  D. N. Willey        Minor upgrade for release 1.0.1 of MacX25.            * */
/* *                                                                                          * */
/* *                                                                                          * */
/* ******************************************************************************************** */
    

/* ******************************************************************************************** */
/* *                                                                     		              * */
/* *	This file is sample code for developers showing the use of the X.25 Interface         * */
/* *    Library.                                                                	          * */
/* *                                                                     		              * */
/* *    It is the second example, and shows the steps necessary to register to receive        * */
/* *    an incoming call, send data over the circut, and then close the SVC and cleanup.      * */
/* *                                                                     		              * */
/* *                                                                     		              * */
/* *    	#	It can be compiled into an MPW Tool using these commands:                     * */
/* *                                                                                          * */
/* *    	C X25_Example_2.c                                                                 * */
/* *        Link -t MPST -c 'MPS ' -o X25_Example_2 		                                 * */
/* *        	X25_Example_2.c.o							                                 * */
/* *        	IPCGlue.o									                                 * */
/* *        	FSDES.o										                                 * */
/* *        	{CLibraries}CRunTime.o						                                 * */
/* *        	{CLibraries}CInterface.o					                                 * */
/* *        	{Libraries}Interface.o						                         		  * */
/* *        	{CLibraries}StdCLib.o                                                         * */
/* *        Rez -a -o :X25_Example_2						                                 * */
/* *            -i  "{MPW}Interfaces:RIncludes:"                                             * */
/* *            X25_Interface_Library.r                                                       * */
/* *                                                                                          * */
/* *                                                                                          * */
/* *        #                                                                                 * */
/* *        #   The X25_Example takes up to three arguments, with the first                   * */
/* *        #   argument being required.                                                      * */
/* *        #                                                                                 * */
/* *        #   The first argument specifies the Call User Data field that will               * */
/* *        #   be used to register to receive incoming calls.                                * */
/* *        #                                                                                 * */
/* *        #   The second and third arguments are strings giving the                         * */
/* *        #   user's Name and Password to be used.  (These arguments are only               * */
/* *        #   required if "guest" login is not allowed to the server.)                      * */
/* *        #                                                                                 * */
/* *        #                                                                                 * */
/* *                                                                                          * */
/* *                                                                                          * */
/* *        #                                                                                 * */
/* *        #	The following command will run this program:                                  * */
/* *        #                                                                                 * */
/* *        #   (First use "MacX25" from the Chooser to                                       * */
/* *        #   select the desired MacX25 Server.)                                            * */
/* *        #                                                                                 * */
/* *                                                                                          * */
/* *        X25_Example_2 "QBF" "User Name" "Password"                                        * */
/* *                                                                                          * */
/* *                                                                                          * */
/* *                                                                                          * */
/* *                                                                                          * */
/* ******************************************************************************************** */






/* ******************************************************************************************** */
/* *                                                                                          * */
/* *    Include all Common Incluce Files and Definitions                                      * */
/* *                                                                                          * */
/* ******************************************************************************************** */

# include <Memory.h>
# include <Events.h>
# include <Quickdraw.h>
# include <stdio.h>

# include <X25_Interface.h>
# include <X25_Packet.h>


/* ******************************************************************************************** */
/* *                                                                                          * */
/* *    Internal Routine Definitions                                                          * */
/* *                                                                                          * */
/* ******************************************************************************************** */

static void		error_print       	( tcb_def 			* 	const	tcb_ptr			, 
								  	  char        		*  	const	message  		);

static OSErr	open_library 		(  												);
static OSErr	open_vccb 			(  												);
static OSErr	open_server 		(  												);
static OSErr	user_validate 		(  												);
static OSErr	open_svc 			(  												);
static OSErr	data_test 			(  												);
static OSErr	data_test_1 		(  												);
static OSErr	invite_clear 		(  												);
	
	
extern char *	c2pstr 				( char 				*			string			);
	

/* ******************************************************************************************** */
/* *                                                                                          * */
/* *    Global Data Area                                                                      * */
/* *                                                                                          * */
/* ******************************************************************************************** */

# define MY_BUF_SZ			512


	tcb_def					tcb;							/* Control Block for one circuit	*/
	tcb_def		*	const	tcb_ptr 		= &tcb;			/* Ptr to Control Block for one VC	*/
	
	char		*			connect_rec_ptr;				/* Pointer to Connection Record		*/
	char		*			user_name_ptr;					/* Pointer to User's Name			*/
	char		*			user_password_ptr;				/* Pointer to User's Password		*/

	char					write_buf     [MY_BUF_SZ+64];	/* Buffer for outgoing packets		*/
	char					read_buf      [MY_BUF_SZ+64];	/* Buffer for incoming packets		*/

	char					aux_write_buf [MY_BUF_SZ+64];	/* Buffer for outgoing data			*/
	char					aux_read_buf  [MY_BUF_SZ+64];	/* Buffer for incoming data			*/



/* ******************************************************************************************** */
/* *                                                                                          * */
/* *	main                                                                                  * */
/* *                                                                                          * */
/* ******************************************************************************************** */

main ( int argc, char *argv[] )
	{
	
	OSErr		exit_code		= X25if_no_error;		/* Holds error returns from routines	*/
	
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Make the Macintosh Toolbox Initialization calls.  Remember that we are running as     | */
/* |    an MPW tool, some initialization calls have already been done by MPW.                 | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	MoreMasters 	(             );
	MoreMasters 	(             );
	InitGraf    	( &qd.thePort );
	InitCursor  	(             );


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Collect the input parameters passed to this program                                   | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	if ( argc < 2 )
		{
		printf ( "Usage: %s Call_Registration_Name User_Name_string User_Password_string \n\n",
				 argv[0] );
		exit ( 1 );
		}
	
	if ( argc >= 2 )
		connect_rec_ptr 	= argv[1];
	else
		connect_rec_ptr 	= "";
		
	
	if ( argc >= 3 )		
		user_name_ptr 		= argv[2];
	else
		user_name_ptr	 	= "";
	
	if ( argc >= 4 )
		user_password_ptr 	= argv[3];
	else
		user_password_ptr 	= "";
	
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Convert the input strings from "C" form to "Pascal" form.                             | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	c2pstr ( connect_rec_ptr   );
	c2pstr ( user_name_ptr     );
	c2pstr ( user_password_ptr );


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Get setup for the processing                                                          | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	printf ( "                                                                              \n" );
	printf ( "#---------------------------------------------------------------------------- \n" );
	printf ( "#                                                                             \n" );


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Call the first routine in the chain                                                   | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	exit_code = open_library ( );
	

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Cleanup and Exit                                                                      | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	
	printf ( "#                                                                             \n" );
	printf ( "#---------------------------------------------------------------------------- \n" );
	printf ( "                                                                              \n" );

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Th-th-th-that's all, folks!                                                           | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

	exit ( exit_code );
	
	}



/* ******************************************************************************************** */
/* *                                                                                          * */
/* *    This routine opens the X.25 Interface Library                                         * */
/* *                                                                                          * */
/* *..........................................................................................* */
/* *                                                                                          * */
/* *    Note: The technique of having a short subroutine which opens "something", calling     * */
/* *          another subroutine to do the next level of processing, and on return closing    * */
/* *          the "something" which had been opened earlier is used throughout this example.  * */
/* *                                                                                          * */
/* *          It emphasises the "paired" operations of opening and closing which are          * */
/* *          required by many of the Library operations.  It is also structurally easy       * */
/* *          to modify one area without affecting other areas (encouraging you to            * */
/* *          experiment with modifying the code).                                            * */
/* *                                                                                          * */
/* *          It does make for a longer program than is really needed, but hopefully it is    * */
/* *          also a clearer program to understand.                                           * */
/* *                                                                                          * */
/* *                                                                                          * */
/* ******************************************************************************************** */

static OSErr open_library (  )
	
	{
		OSErr		the_result;					/* Holds error returns from this routine 		*/
		OSErr		exit_code;					/* Holds returns from next lower level routines	*/
	
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Initialize the X.25 Interface Library                                                 | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
	
		the_result = X25_Open ( nil, application_type );
	
		if ( the_result != X25if_no_error )
			{
			printf ( "### X25 Library Initialization Failed: return_code = 0x%04X \n", the_result );
			return ( the_result );
			}
		
		printf ( "#   X25 Library Opened \n" );
		fflush ( stdout );
	
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |	The Library is now open.  Go to the next subroutine in the processing chain.          | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
	
		exit_code = open_vccb ( );
		
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Close the X.25 Library.                                                               | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
	
		the_result = X25_Close ( );
	
		if ( the_result != X25if_no_error )
			printf ( "### X25 Library Close Failed: return_code = 0x%04X \n", the_result );
		else
			printf ( "#   X25 Library Closed \n" );
		fflush ( stdout );
	
		
		
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    All done, return.                                                                     | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		return ( exit_code );
	}



/* ******************************************************************************************** */
/* *                                                                                          * */
/* *    This routine first initializes our "tcb" (Transmission Control Block). It then        * */
/* *    makes the Library call to initialize the "vccb" (Virtual Circuit Control Block),      * */
/* *    and then calls the next level of processing.  On return, it closes the "vccb".        * */
/* *                                                                                          * */
/* ******************************************************************************************** */

static OSErr open_vccb (  )
	
	{
		OSErr		the_result;					/* Holds error returns from this routine 		*/
		OSErr		exit_code;					/* Holds returns from next lower level routines	*/
	
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Clear the tcb area before use                                                         | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
	
		{
			register	char	* 	to_ptr	= (char*)tcb_ptr;
			register	int			count	= sizeof ( tcb_def );
			
			while ( count-- > 0 )
				*to_ptr++ = 0;
		}
	
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Initialize the control block for a new circuit.                                       | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |    Note: The function return value and the value stored in tcb_ptr -> return_code        | */
/* |          are identical in all cases EXCEPT the case where the address of a Circuit       | */
/* |          is NULL. If you are confident that your code cannot make this error, you        | */
/* |          need only check the tcb_ptr -> return_code value.                               | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
	
		the_result = X25_vccb_Open ( tcb_ptr, nil, MY_BUF_SZ );
	
		if ( the_result != X25if_no_error )
			{
			error_print ( tcb_ptr, "vccb Initialization Failed:" );
			return ( the_result );
			}
		
		printf ( "#   vccb Opened \n" );
		fflush ( stdout );
	
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |	Now go to the next level of processing which will open a connection to the            | */
/* |    X.25 Server.                                                                          | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
	
		exit_code = open_server ( );
	
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Close the vccb.                                                                       | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
	
		the_result = X25_vccb_Close ( tcb_ptr );
	
		if ( the_result != X25if_no_error )
			printf ( "vccb Close Failed: return_code = 0x%04X \n", the_result );
		else
			printf ( "#   vccb Closed \n" );
		fflush ( stdout );
	
		
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    All done, return.                                                                     | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		return ( exit_code );
	}



/* ******************************************************************************************** */
/* *                                                                                          * */
/* *    This routine handles the connection to the specified X.25 Server.  Once connection    * */
/* *    is established, another subroutine is called for the next level of processing.        * */
/* *                                                                                          * */
/* ******************************************************************************************** */

static OSErr open_server (  )
	
	{
		OSErr		the_result;					/* Holds error returns from this routine 		*/
		OSErr		exit_code;					/* Holds returns from next lower level routines	*/
	
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Obtains the server's AppleTalk Name and Zone as last selected with the Chooser.       | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
	
		the_result = X25_Chooser ( tcb_ptr );
	
		if ( the_result != X25if_no_error )
			{
			error_print ( tcb_ptr, "Access to Chooser Failed:" );
			return ( the_result );
			}
		
		
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Get the Server's Name and Zone and print this information.                            | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
	
		{
			char		server_name [256]	= "\p";
			char		server_zone [256]	= "\p";
		
			the_result = X25_Get_Server_ID ( tcb_ptr, server_name, server_zone );
		
			if ( the_result != X25if_no_error )
				{
				error_print ( tcb_ptr, "Retreival of Server Name/Zone Failed:" );
				return ( the_result );
				}
			
			printf ( "#   Server Name:                        = '%P' \n", server_name );
			printf ( "#   Server Zone:                        = '%P' \n", server_zone );
			fflush ( stdout );
		}
		
		
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Setup the tcb for the server connection call.                                         | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		tcb_ptr -> aux_write_data.data_ptr		= aux_write_buf;
		tcb_ptr -> aux_write_data.max_length	= sizeof ( aux_write_buf );

		tcb_ptr -> write_data.data_ptr			= write_buf;
		tcb_ptr -> write_data.max_length		= sizeof ( write_buf );

		tcb_ptr -> read_data.data_ptr			= read_buf;
		tcb_ptr -> read_data.req_length			= sizeof ( read_buf );
		tcb_ptr -> read_data.max_length			= sizeof ( read_buf );

		tcb_ptr -> aux_read_data.data_ptr		= aux_read_buf;
		tcb_ptr -> aux_read_data.req_length		= sizeof ( aux_read_buf );
		tcb_ptr -> aux_read_data.max_length		= sizeof ( aux_read_buf );

		tcb_ptr -> write_data.IF_asynchronous	= false;
		tcb_ptr -> write_data.timeout			= timeout_sec ( 10 );

		tcb_ptr -> read_data.IF_asynchronous	= false;
		tcb_ptr -> read_data.timeout			= timeout_sec ( 10 );

		tcb_ptr -> ref_con						= nil;


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Connect to the specified X.25 Server.                                                 | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
	
		the_result = X25_Server_Connect ( tcb_ptr, connect_via_best );
	
		if ( the_result != X25if_no_error )
			{
			error_print ( tcb_ptr, "Connection to Server Failed:" );
			return ( the_result );
			}
		
		printf ( "#   Server Connection Opened \n" );
		fflush ( stdout );
		
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |	Now connected to the X.25 Server.  Go to a subroutine to make an X.25 SVC connection  | */
/* |    and exchange some data.                                                               | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
	
		exit_code = user_validate ( );
		
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |	All done, need to disconnect from the X.25 Server.                                    | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
	
		the_result = X25_Server_Discnct ( tcb_ptr );
	
		if ( the_result != X25if_no_error )
			error_print ( tcb_ptr, "Disconnection from Server Failed:" );
		else
			printf ( "#   Server Connection Closed \n" );
		fflush ( stdout );
		
		
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    All done, return.                                                                     | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		return ( exit_code );
	}



/* ******************************************************************************************** */
/* *                                                                                          * */
/* *    This routine validates that the user is allowed to use the specified X.25 Server.     * */
/* *                                                                                          * */
/* ******************************************************************************************** */

static OSErr user_validate (  )
	
	{
		OSErr		the_result;					/* Holds error returns from this routine 		*/
		OSErr		exit_code;					/* Holds returns from next lower level routines	*/
	
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Set the supplied user name and password                                               | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		printf ( "#   User Name:                          = '%P' \n", user_name_ptr     );
		printf ( "#   User Password:                      = '%P' \n", user_password_ptr );
	
		the_result = X25_Set_User_ID ( tcb_ptr, user_name_ptr     ,
		                                        user_password_ptr );
	
		if ( the_result != X25if_no_error )
			{
			error_print ( tcb_ptr, "Could Not Set User ID:" );
			return ( the_result );
			}


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Setup the tcb for the validation call.                                                | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		tcb_ptr -> aux_write_data.data_ptr		= aux_write_buf;
		tcb_ptr -> aux_write_data.max_length	= sizeof ( aux_write_buf );

		tcb_ptr -> write_data.data_ptr			= write_buf;
		tcb_ptr -> write_data.max_length		= sizeof ( write_buf );

		tcb_ptr -> read_data.data_ptr			= read_buf;
		tcb_ptr -> read_data.req_length			= sizeof ( read_buf );
		tcb_ptr -> read_data.max_length			= sizeof ( read_buf );

		tcb_ptr -> aux_read_data.data_ptr		= aux_read_buf;
		tcb_ptr -> aux_read_data.req_length		= sizeof ( aux_read_buf );
		tcb_ptr -> aux_read_data.max_length		= sizeof ( aux_read_buf );

		tcb_ptr -> write_data.IF_asynchronous	= false;
		tcb_ptr -> write_data.completion		= nil;
		tcb_ptr -> write_data.timeout			= timeout_sec ( 5 );

		tcb_ptr -> read_data.IF_asynchronous	= false;
		tcb_ptr -> read_data.completion			= nil;
		tcb_ptr -> read_data.timeout			= timeout_sec ( 5 );

		tcb_ptr -> ref_con						= nil;


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Make the validation call.                                                             | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
	
		the_result = X25_Server_Authorize ( tcb_ptr );
	
		if ( the_result != X25if_no_error )
			{
			error_print ( tcb_ptr, "Authorization to Server Failed:" );
			return ( the_result );
			}
		
		printf ( "#   Server Authorization Made \n" );
		fflush ( stdout );
		
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |	Now connected to the X.25 Server.  Go to a subroutine to make an X.25 SVC connection  | */
/* |    and exchange some data.                                                               | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
	
		exit_code = open_svc ( );
		
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    All done, return.                                                                     | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		return ( exit_code );
	}



/* ******************************************************************************************** */
/* *                                                                                          * */
/* *	open_svc		                                                                      * */
/* *                                                                                          * */
/* *------------------------------------------------------------------------------------------* */
/* *                                                                                          * */
/* *    This routine (attempts to) establishes an X.25 SVC to the selected address.  After    * */
/* *    the connection is made, it calls another routine to ship data in several different    * */
/* *    modes.  On return, it closes the X.25 SVC and returns.                                * */
/* *                                                                                          * */
/* ******************************************************************************************** */

static OSErr	open_svc       	(  )
	
	{
		OSErr		the_result;					/* Holds error returns from this routine 		*/
		OSErr		exit_code;					/* Holds returns from next lower level routines	*/

		char		connection_record [256];


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Build the X.25 Connection Record.                                                     | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		sprintf ( connection_record, "L -D%P", connect_rec_ptr );
		c2pstr ( connection_record );
		
		printf ( "#   Connection Record:                  = '%P' \n", connection_record );
		fflush ( stdout );
		
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Setup the tcb for the "X25_Connect" call.                                             | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	"aux_write_data.data_ptr" is set to point to the supplied Connection          | */
/* |            Record.                                                                       | */
/* |                                                                                          | */
/* |            Also note that that the timeout is set to "timeout_infinite" because the      | */
/* |            this routine won't complete until somebody calls us.                          | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		tcb_ptr -> aux_write_data.data_ptr		= connection_record;

		tcb_ptr -> write_data.data_ptr			= write_buf;
		tcb_ptr -> write_data.max_length		= sizeof ( write_buf );

		tcb_ptr -> read_data.data_ptr			= read_buf;
		tcb_ptr -> read_data.req_length			= sizeof ( read_buf );
		tcb_ptr -> read_data.max_length			= sizeof ( read_buf );

		tcb_ptr -> aux_read_data.data_ptr		= aux_read_buf;
		tcb_ptr -> aux_read_data.req_length		= sizeof ( aux_read_buf );
		tcb_ptr -> aux_read_data.max_length		= sizeof ( aux_read_buf );

		tcb_ptr -> write_data.IF_asynchronous	= false;
		tcb_ptr -> write_data.completion		= nil;
		tcb_ptr -> write_data.timeout			= timeout_infinite;

//		tcb_ptr -> read_data.IF_asynchronous	= false;
//		tcb_ptr -> read_data.completion			= nil;
//		tcb_ptr -> read_data.timeout			= timeout_infinite;
//
		tcb_ptr -> ref_con						= nil;

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |	Make the X.25 SVC Connection.                                                         | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	X.25_Connect will take an ASCII text string as the call placement record,     | */
/* |            build a call request packet and send it, then read the returned packet        | */
/* |            to see if it was a call accepted packet or a clear packet (or something       | */
/* |            else).                                                                        | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		the_result = X25_Connect ( tcb_ptr );

		if ( the_result == X25if_call_cleared )
			{
			printf ( "# \n" );
			printf ( "### %P \n", tcb_ptr -> aux_read_data.data_ptr );
			}
		
		if ( the_result != X25if_no_error )
			{
			error_print ( tcb_ptr, "X25 SVC Connection Failed:" );
			return ( the_result );
			}
		
		printf ( "#   X.25 SVC Connection Opened \n" );
		fflush ( stdout );
	
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |	Now have an X.25 SVC Connection.  Go to another subroutine which will exercise the    | */
/* |    path by exchanging data with the remote end.                                          | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		exit_code = data_test ( );


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Setup the tcb for the "X25_Discnct" call.                                             | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		tcb_ptr -> aux_write_data.data_ptr		= aux_write_buf;
		tcb_ptr -> aux_write_data.max_length	= sizeof ( aux_write_buf );

		tcb_ptr -> write_data.data_ptr			= write_buf;
		tcb_ptr -> write_data.max_length		= sizeof ( write_buf );

		tcb_ptr -> read_data.data_ptr			= read_buf;
		tcb_ptr -> read_data.req_length			= sizeof ( read_buf );
		tcb_ptr -> read_data.max_length			= sizeof ( read_buf );

		tcb_ptr -> aux_read_data.data_ptr		= aux_read_buf;
		tcb_ptr -> aux_read_data.req_length		= sizeof ( aux_read_buf );
		tcb_ptr -> aux_read_data.max_length		= sizeof ( aux_read_buf );

		tcb_ptr -> write_data.IF_asynchronous	= false;
		tcb_ptr -> write_data.completion		= nil;
		tcb_ptr -> write_data.timeout			= timeout_sec ( 5 );

		tcb_ptr -> ref_con						= nil;
		
		aux_write_buf[0] = '\0';				/* Force a zero length string					*/


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |	Clear the X.25 SVC Connection.                                                        | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	This isn't really necessary.  When you do the Server Disconnect, the          | */
/* |            Server Agent will notify the X.25 code who will then ensure that the          | */
/* |            X.25 SVC is closed.  However, its better to do things by the book in          | */
/* |            case of future changes to the software.                                       | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		the_result = X25_Discnct ( tcb_ptr );
	
		if ( the_result != X25if_no_error )
			error_print ( tcb_ptr, "X25 SVC Disconnection Failed:" );
		else
			printf ( "#   X.25 SVC Connection Closed \n" );
		fflush ( stdout );
	

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    All done, return.                                                                     | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		return ( exit_code );
	}




/* ******************************************************************************************** */
/* *                                                                                          * */
/* *	data_test                                                                             * */
/* *                                                                                          * */
/* *------------------------------------------------------------------------------------------* */
/* *                                                                                          * */
/* *    At this point we have an X.25 SVC to a remote entity.  We may now make reads and      * */
/* *    writes of arbitrary data to our heart's content.  This routine will several           * */
/* *    different routines to send and receive data to demonstrate the various modes of       * */
/* *    I/O operation.                                                                        * */
/* *                                                                                          * */
/* *    For all these examples, we assume that the remote entity is an "echo process", which  * */
/* *    will simply return each of our packets to us. Each write will be followed by a        * */
/* *    read, and in these particular circumstances, we can consider it an error if we        * */
/* *    don't get back exactly what we sent.                                                  * */
/* *                                                                                          * */
/* ******************************************************************************************** */

static OSErr	data_test       	(  )
	
	{
		OSErr		exit_code;							/* Holds error returns from routines	*/


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Synchronous, blocking call with timeouts                                              | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		exit_code = data_test_1 ( );
		
		if ( exit_code != X25if_no_error )
			return ( exit_code );


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Send an "invitation to clear" to the remote end                                       | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		exit_code = invite_clear ( );
		
		if ( exit_code != X25if_no_error )
			return ( exit_code );
	

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Start a 20 sec time and let it burn down before we force a disconnect.                | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		{
			long		quit_time;
			
			quit_time = TickCount ( ) + 20 * 60;
			
			while ( TickCount ( ) < quit_time ) {
				
				//	should do a read and see if a clear request packet came in.
				//	if one did, then we can get out of this loop.
				//
				//	if only there were more hours in the day...
				
				}
				
		}

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    All Done, Return                                                                      | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		return ( X25if_no_error );
	}

	

/* ******************************************************************************************** */
/* *                                                                                          * */
/* *	data_test_1		( synchronous, blocking, timeouts )                                   * */
/* *                                                                                          * */
/* *------------------------------------------------------------------------------------------* */
/* *                                                                                          * */
/* *    This test uses a "synchronous", "blocking" write, withe "timeouts", followed by the   * */
/* *    same type of read".                                                                   * */
/* *                                                                                          * */
/* *    A "Non-blocking write means if the circuit is flow-controlled in the outbound         * */
/* *    direction, the X25_Write call will return WITHOUT ERROR but with ZERO BYTES WRITTEN.  * */
/* *    In this example, it is highly unlikely that a write will be flow-controlled           * */
/* *    (especially the first write after a call connection), but use your imagination.       * */
/* *                                                                                          * */
/* ******************************************************************************************** */

static 	OSErr		data_test_1       	(  )
	
	{

		OSErr		the_result;							/* Holds error returns from routines	*/
		
		int			count			= 0;
		
		char		msg []			= "\pThe quick brown fox jumped over the lazy dog's back.";

		char		xmit_data [256];					/* The data that we are to send			*/


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Do a loop of 9 packets to be sent.                                                    | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		for ( count = 1; count < 10; count++ ) {
		

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Build the message to be sent.                                                         | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

			sprintf ( xmit_data, "%02d %P \r\n", count, msg );
			c2pstr ( xmit_data );
			

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Setup the tcb for the "X25_encode_data" call, then make the encode call.              | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	The first line of code sets the pointer of the data to be encoded to be       | */
/* |            the address of the Pascal string.  The second line of code sets the length    | */
/* |            to be that of the string (string length byte plus 1).                         | */
/* |                                                                                          | */
/* |            The second two lines of code set the buffer where the encoded packet is       | */
/* |            to be built.  (These values are already set, but for this example we do       | */
/* |            it again.)                                                                    | */
/* |                                                                                          | */
/* |            The encode routine will set the "write_data.req_length" to the correct        | */
/* |            value for the X25_Write call.                                                 | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
			
			tcb_ptr -> aux_write_data.data_ptr     	= &(xmit_data[0]);	
			
			tcb_ptr -> aux_write_data.M_bit		    = false;
			tcb_ptr -> aux_write_data.D_bit		    = false;
			tcb_ptr -> aux_write_data.Q_bit		    = false;

			tcb_ptr -> write_data.data_ptr 			= write_buf;
			tcb_ptr -> write_data.max_length 		= sizeof ( write_buf );
	
			the_result = X25_encode_text ( tcb_ptr );
		
			if ( the_result != X25if_no_error )
				{
				error_print ( tcb_ptr, "Data_Test_1: Encode of Data Failed:" );
				return ( the_result );
				}
		
		
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Setup the tcb for the "X25_Write" call.                                               | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	The "write_buffer.data_ptr" was set for the encode call and is still intact.  | */
/* |            "write_buffer.req_length" was set by the encode call.                         | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
	
			tcb_ptr -> write_data.timeout    		= timeout_sec ( 5 );
			tcb_ptr -> write_data.IF_asynchronous 	= false;
			tcb_ptr -> write_data.IF_more         	= false;
			tcb_ptr -> write_data.IF_expedited    	= false;
		
			tcb_ptr -> write_data.completion    	= nil;		/* not used on sync calls		*/
		
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Make the "X25_Write" call.                                                            | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |	Note:	We loop until either the data is written, the write call fails, or until      | */
/* |            the number of attempts exceeds a limit (a poor man's timeout).  In real       | */
/* |            life, we'd find something useful to do with our time in between polling       | */
/* |            the X25_Write.                                                                | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
			
			the_result = X25_Write ( tcb_ptr );
			
			if ( the_result != X25if_no_error )
				{
				error_print ( tcb_ptr, "Data_Test_1 X.25 Write Failed:" );
				X25_Server_Discnct (tcb_ptr);								// KLUDGE ????
				return ( the_result );
				}
		
		
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    End of the "for" loop.                                                                | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

			}
			

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    All done, return.                                                                     | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		return ( X25if_no_error );
	}



/* ******************************************************************************************** */
/* *                                                                                          * */
/* *	invite_clear		                                                                  * */
/* *                                                                                          * */
/* *------------------------------------------------------------------------------------------* */
/* *                                                                                          * */
/* *    This uses a "synchronous", "blocking" write, with "timeouts".                         * */
/* *                                                                                          * */
/* ******************************************************************************************** */

static 	OSErr		invite_clear       	(  )
	
	{

		OSErr		the_result;							/* Holds error returns from routines	*/
		
		char		xmit_data []		= 0x01;			/* The data that we are to send			*/


/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Setup the tcb for the "X25_encode_data" call, then make the encode call.              | */
/* |                                                                                          | */
/* |..........................................................................................| */
/* |                                                                                          | */
/* |    Note that the Q-bit is set on                                                         | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */
		
		tcb_ptr -> aux_write_data.data_ptr     	= xmit_data;	
		tcb_ptr -> aux_write_data.req_length   	= sizeof ( xmit_data );	
		
		tcb_ptr -> aux_write_data.M_bit		    = false;
		tcb_ptr -> aux_write_data.D_bit		    = false;
		tcb_ptr -> aux_write_data.Q_bit		    = true;

		tcb_ptr -> write_data.data_ptr 			= write_buf;
		tcb_ptr -> write_data.max_length 		= sizeof ( write_buf );

		the_result = X25_encode_data ( tcb_ptr );
	
		if ( the_result != X25if_no_error )
			{
			error_print ( tcb_ptr, "invite_clear: Encode of Data Failed:" );
			return ( the_result );
			}
	
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Setup the tcb and make the "X25_Write" call.                                          | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		tcb_ptr -> write_data.timeout    		= timeout_sec ( 5 );
		tcb_ptr -> write_data.IF_asynchronous 	= false;
		tcb_ptr -> write_data.IF_more         	= false;
		tcb_ptr -> write_data.IF_expedited    	= false;
	
		the_result = X25_Write ( tcb_ptr );
		
		if ( the_result != X25if_no_error )
			{
			error_print ( tcb_ptr, "invite_clear: X.25 Write Failed:" );
			X25_Server_Discnct (tcb_ptr);
			return ( the_result );
			}
	
	
/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    All done, return.                                                                     | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		return ( X25if_no_error );
	}



/* ******************************************************************************************** */
/* *                                                                                          * */
/* *    Format and printout any errors that occur                                             * */
/* *                                                                                          * */
/* ******************************************************************************************** */

static void	error_print       	( tcb_def 			* 	const	tcb_ptr			, 
								  char        		*  	const	message  		)
	
	{
		OSErr						the_result;
		char						msg_area [256]	= "\p";

		printf ( "# \n" );

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Do a simple printout of where the failure occured along with the return and status    | */
/* |    codes.                                                                                | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		printf ( "### %s return_code = 0x%04X, status_code = 0x%04X\n" , 
				 message                                               ,
		         tcb_ptr -> return_code & 0x0000FFFF                   ,
		         tcb_ptr -> status_code & 0x0000FFFF                   );
	

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    Call the Library routine to generate an decoded message and print this information.   | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		the_result = X25_error_decode ( tcb_ptr -> return_code , 
		                                tcb_ptr -> status_code , 
										msg_area               );
		
		printf ( "### %P \n", msg_area );
	

/* +------------------------------------------------------------------------------------------+ */
/* |                                                                                          | */
/* |    All done, return.                                                                     | */
/* |                                                                                          | */
/* +------------------------------------------------------------------------------------------+ */

		printf ( "# \n" );
		fflush ( stdout );

		return;
	}
	
	

/* ******************************************************************************************** */
/* *                                                                                          * */
/* *    End of Program                                                                        * */
/* *                                                                                          * */
/* ******************************************************************************************** */

